#ifndef	__FSTYPES_H
#define __FSTYPES_H
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <memory.h>
#include <unistd.h>
#include <netdb.h>

#define	WINAPI
#define FCNET_SDK

typedef long long				WPARAM;
typedef	long long				LPARAM;
typedef void *					HFSHTTP;
typedef void *					HFSSEARCH;
typedef void *					HFSCLIENT;
typedef void *					HFSTHREAD;

typedef signed char		int8_t;
typedef unsigned char   uint8_t;
typedef short			int16_t;
typedef unsigned short  uint16_t;
typedef int				int32_t;
typedef unsigned		uint32_t;

typedef pthread_mutex_t			MUTEX;
#define INIT_MUTEX(x)			pthread_mutex_init(x, NULL)
#define RELEASE_MUTEX(x)		pthread_mutex_destroy(x)
#define LOCK_MUTEX(x)			pthread_mutex_lock(x)
#define UNLOCK_MUTEX(x)			pthread_mutex_unlock(x)

#define	THREAD_FUNC_TO			void *(*)(void *)
#define	THREAD_FUNC				void * (*func)(void *)
typedef void* (*THREADFUNC)(void*);
typedef long long				DWORD;

typedef struct MSG
{
	u_int32_t	message;
	WPARAM		wParam;
	LPARAM		lParam;
}MSG;

typedef struct tagFRAME_HEAD{
	int			type; // MF_AUDIO or MF_VIDEO
	long long	timeStamp;
	uint32_t    videoFrameRate;
	uint32_t	length; // Frame length
	uint32_t	isKeyFrame;
	uint32_t	videoBitRate;
	uint64_t	reserve;
	unsigned char*	frame;
} FRAME_HEAD, *LPFRAME_HEAD;

// Param
//      type		:MF_AUDIO  Audio frame
//					 MF_VIDEO  Video frame
//      frame_head	:Frame data header address, see LPFRAME_HEAD for detail
//      udata		:Parameter passed by fs_net_open()
//      utype		:Not in use
typedef void (*FRAME_CB)(int type, LPFRAME_HEAD frame_head, void* udata, int utype);

// Param
//      type		:Always 1 currently, not in use
//      buf			:Search result buffer, see SEARCH_S for detail
//      length		:Search result buffer length
//      udata		:Parameter passed by fs_search_devices()
typedef int (*SEARCH_CB)(int type, void *buf, int length, void *userdata);

// Param
//      msg			:Message id
//      subMsg		:Sub message id
//      param		:Message parameter
//		udata		:Parameter passed by fs_net_open()
//      utype		:Not in use
typedef void (*MSG_CB2)(int msg, int subMsg, void *param, void *userdata, int utype);

// Callback message
enum
{
	NET_MSG_CONNECT, // Connect callback message
	NET_MSG_CTRL = 8, // Control command callback message
	NET_MSG_TALK_OPEN = 9, // Talk open message
	NET_MSG_TALK_CLOSE = 10, // Talk close message
	
};

// Callback sub message
enum
{
	NETMSG_CONNECT_SUC, //Connect success
	NETMSG_CONNECT_IPORPORTERROR, // IP or port error
	NETMSG_CONNECT_CONNECTERROR, // Net error
	NETMSG_TALK_SUC = 20, // Open talk success
	NETMSG_TALK_ERROR, // Open talk fail
	
	NETMSG_CTRL_CAPTURE = 50 // Snap result
};

typedef enum {
	DT_NIL = 0,
	DT_MJPEG,
	DT_H264,
	DT_UDT_UDP,
	DT_UDT_TCP,
	DT_OTHER_HX,
	DT_UDT_MUTIL_CHANNEL_TUTK
} DEVICE_TYPE_T;

typedef enum {
	MF_UNKNOW = 0x100,
	MF_YUYV422,
	MF_YUV420P,
	MF_PCM,
	MF_IMAADPCM,
	MF_H264,
	MF_MJPEG,
	MF_G726,

	MF_VIDEO,
	MF_AUDIO
} MEDIA_FORMAT_T;


// 
typedef enum{
	VCIF_STARTVIDEO,
	VCIF_STARTAUDIO,
	VCIF_STARTTALK,
	
	VCIF_TALKDATA=4, // send talk data, audio data is passed by lParam
	
	VCIF_STOPVIDEO,
	VCIF_STOPAUDIO,
	VCIF_STOPTALK,

	VCIF_CONTROL = 12,
}VCLIENT_INTERFACE;


// lParam of VCIF_CONTROL
typedef enum{
	// PT control command param
	VCIF_CONTROL_UP,
	VCIF_CONTROL_DOWN,
	VCIF_CONTROL_LEFT,
	VCIF_CONTROL_RIGHT,
	VCIF_CONTROL_LEFTUP,
	VCIF_CONTROL_RIGHTUP,
	VCIF_CONTROL_LEFTDOWN,
	VCIF_CONTROL_RIGHTDOWN,
	VCIF_CONTROL_STOP = 9,

	// Snap command param
	VCIF_CONTROL_CAPTURE = 23,
}VCLIENTCONTROL;

typedef struct tagNETPARAM{
	long    groupid;	// Key id use to distinguish different camera, you can use current system time as an key id
	char	ip[128]; // IP or url of the camera you want to connect
	int		port; // Media port of camera
	char	username[20]; // User name of camera
	char	passwd[20]; // password of camera
	short	streamType; // 0=Main stream 1=Sub stream
	short	webPort; // http port of camera
}NETPARAM, *LPNETPARAM;

#define ATTRIBUTE_PACKED __attribute__((packed))
typedef struct {
	char		id[13];  // Camera id, mac address of camera now
	char		name[21]; // Camera name
	uint32_t	ip; // IP address of camera
	uint32_t	mask; // Net mask of camera
	uint32_t	gateway; // Gate way of camera
	uint32_t	dns; // DNS of camera
	char		type; // 0=MJPEG  1=H264
	char		isP2P; // Is P2P is supported by this camera
	short		mediaPort; // Media port(Used to trans video and audio stream) of camera
	uint32_t	sys_ver; // System firmware version
	uint32_t	app_ver; // APP firmware version
	uint16_t	port; // http port(Used to trans command) of camera
	uint8_t		dhcp_enabled; // Obtain ip address from dhcp server or not
	char		uid[64]; // The uid of camera if the camera support P2P
} ATTRIBUTE_PACKED SEARCH_S;
#endif
